@@ -10,15 +10,16 @@ import requests
 
 from requests.adapters import HTTPAdapter
 from requests.packages.urllib3.poolmanager import PoolManager
-import ssl
 
+import platform
+import ssl
+import sys
 
 class MPSSLAdapter(HTTPAdapter):
     def init_poolmanager(self, connections, maxsize, block=False):
         self.poolmanager = PoolManager(num_pools=connections,
                                        maxsize=maxsize,
-                                       block=block,
-                                       ssl_version=ssl.PROTOCOL_TLSv1)
+                                       block=block)
 
 class MPException(Exception):
     def __init__(self, value):
@@ -32,13 +33,30 @@ class MPInvalidCredentials(MPException):
 
 
 class MP(object):
-    version = "0.3.1"
+    version = "1.2.0"
     __access_data = None
+    __ll_access_token = None
     __sandbox = False
+    platform_id = None
+    integrator_id = None
+    corporation_id = None
+
+    def __init__(self, *args):
+        """
+        Instantiate MP with credentials:
+        mp = mercadopago.MP(client_id, client_secret)
+
+        Instantiate MP with Long Live Access Token:
+        mp = mercadopago.MP(ll_access_token)
+        """
+        if len(args) == 2:
+            self.__client_id = args[0]
+            self.__client_secret = args[1]
+        elif len(args) == 1:
+            self.__ll_access_token = args[0]
+        else:
+            raise MPInvalidCredentials(None)
 
-    def __init__(self, client_id, client_secret):
-        self.__client_id = client_id
-        self.__client_secret = client_secret
         self.__rest_client = self.__RestClient(self)
 
     def sandbox_mode(self, enable=None):
@@ -48,6 +66,9 @@ class MP(object):
         return self.__sandbox
 
     def get_access_token(self):
+        if not self.__ll_access_token is None:
+            return self.__ll_access_token
+
         app_client_values = {
                            "client_id": self.__client_id,
                            "client_secret": self.__client_secret,
@@ -62,19 +83,24 @@ class MP(object):
         else:
             raise MPInvalidCredentials(str(access_data))
 
+    def set_platform_id(self, platform_id):
+        self.platform_id = platform_id
+
+    def set_integrator_id(self, integrator_id):
+        self.integrator_id = integrator_id
+
+    def set_corporation_id(self, corporation_id):
+        self.corporation_id = corporation_id
+
     def get_payment(self, id):
         """
         Get information for specific payment
-        
+
         @param id
         @return json
         """
 
-        access_token = self.get_access_token()
-
-        uri_prefix = "/sandbox" if self.__sandbox else ""
-
-        payment_info = self.__rest_client.get(uri_prefix+"/collections/notifications/"+id+"?access_token="+access_token)
+        payment_info = self.__rest_client.get("/v1/payments/"+id)
         return payment_info
 
     def get_payment_info(self, id):
@@ -83,150 +109,116 @@ class MP(object):
     def get_authorized_payment(self, id):
         """
         Get information for specific authorized payment
-        
+
         @param id
         @return json
-        """    
-
-        try:
-            access_token = self.get_access_token()
-        except Exception,e:
-            raise e
-        
-        authorized_payment_info = self.__rest_client.get("/authorized_payments/"+id+"?access_token="+access_token)
+        """
+        authorized_payment_info = self.__rest_client.get("/authorized_payments/"+id)
         return authorized_payment_info
 
-    
+
     def refund_payment(self, id):
         """
         Refund accredited payment
-        
+
         @param id
         @return json
         """
-    
-        access_token = self.get_access_token()
 
-        refund_status = {"status":"refunded"}
-
-        response = self.__rest_client.put("/collections/"+id+"?access_token="+access_token, refund_status)
+        refund_status = {}
+        response = self.__rest_client.post("/v1/payments/"+id+"/refunds", refund_status)
         return response
 
-    
+
     def cancel_payment(self, id):
         """
         Cancel pending payment
-        
+
         @param id
         @return json
         """
-    
-        access_token = self.get_access_token()
 
         cancel_status = {"status":"cancelled"}
-
-        response = self.__rest_client.put("/collections/"+id+"?access_token="+access_token, cancel_status)
+        response = self.__rest_client.put("/v1/payments/"+id, cancel_status)
         return response
 
 
     def cancel_preapproval_payment(self, id):
         """
         Cancel preapproval payment
-        
+
         @param id
         @return json
-        """    
-
-        try:
-            access_token = self.get_access_token()
-        except Exception,e:
-            raise e
+        """
 
         cancel_status = {"status":"cancelled"}
-        
-        response = self.__rest_client.put("/preapproval/"+id+"?access_token="+access_token, cancel_status)
+        response = self.__rest_client.put("/preapproval/"+id, cancel_status)
         return response
-    
+
 
     def search_payment(self, filters, offset=0, limit=0):
         """
         Search payments according to filters, with pagination
-        
+
         @param filters
         @param offset
         @param limit
         @return json
         """
 
-        access_token = self.get_access_token()
-
-        filters["access_token"] = access_token
         filters["offset"] = offset
         filters["limit"] = limit
 
-        uri_prefix = "/sandbox" if self.__sandbox else ""
-
-        payment_result = self.__rest_client.get(uri_prefix+"/collections/search", filters)
+        payment_result = self.__rest_client.get("/v1/payments/search", filters)
         return payment_result
 
     def create_preference(self, preference):
         """
         Create a checkout preference
-        
+
         @param preference
         @return json
         """
 
-        access_token = self.get_access_token()
-
-        preference_result = self.__rest_client.post("/checkout/preferences?access_token="+access_token, preference)
+        preference_result = self.__rest_client.post("/checkout/preferences", preference)
         return preference_result
 
 
     def update_preference(self, id, preference):
         """
         Update a checkout preference
-        
+
         @param id
         @param preference
         @return json
         """
 
-        access_token = self.get_access_token()
-
-        preference_result = self.__rest_client.put("/checkout/preferences/"+id+"?access_token="+access_token, preference)
+        preference_result = self.__rest_client.put("/checkout/preferences/"+id, preference)
         return preference_result
 
 
     def get_preference(self, id):
         """
         Update a checkout preference
-        
+
         @param id
         @param preference
         @return json
         """
-        access_token = self.get_access_token()
-
-        preference_result = self.__rest_client.get("/checkout/preferences/"+id+"?access_token="+access_token)
+        preference_result = self.__rest_client.get("/checkout/preferences/"+id)
         return preference_result
-    
+
 
     def create_preapproval_payment(self, preapproval_payment):
         """
         Create a preapproval payment
-        
+
         @param preapproval_payment
         @return json
         """
-        try:
-            access_token = self.get_access_token()
-        except Exception,e:
-            raise e
-
-        preapproval_payment_result = self.__rest_client.post("/preapproval?access_token="+access_token, preapproval_payment)
+        preapproval_payment_result = self.__rest_client.post("/preapproval", preapproval_payment)
         return preapproval_payment_result
-    
+
     def get_preapproval_payment(self, id):
         """
         Get a preapproval payment
@@ -235,14 +227,9 @@ class MP(object):
         @return json
 
         """
-        try:
-            access_token = self.get_access_token()
-        except Exception,e:
-            raise e
-        
-        preapproval_payment_result = self.__rest_client.get("/preapproval/"+id+"?access_token="+access_token)
+        preapproval_payment_result = self.__rest_client.get("/preapproval/"+id)
         return preapproval_payment_result
-    
+
     def update_preapproval_payment(self, id, preapproval_payment):
         """
         Update a preapproval payment
@@ -251,14 +238,9 @@ class MP(object):
         @return json
 
         """
-        try:
-            access_token = self.get_access_token()
-        except Exception,e:
-            raise e
-        
-        preapproval_payment_result = self.__rest_client.put("/preapproval/"+id+"?access_token="+access_token, preapproval_payment)
+        preapproval_payment_result = self.__rest_client.put("/preapproval/"+id, preapproval_payment)
         return preapproval_payment_result
-    
+
     def get(self, uri, params=None, authenticate=True):
         """
         Generic resource get
@@ -270,16 +252,9 @@ class MP(object):
         if params is None:
             params = {}
 
-        if authenticate:
-            try:
-                access_token = self.get_access_token()
-                params["access_token"] = access_token
-            except Exception,e:
-                raise e
-
         result = self.__rest_client.get(uri, params)
         return result
-    
+
     def post(self, uri, data, params=None):
         """
         Generic resource post
@@ -291,15 +266,9 @@ class MP(object):
         if params is None:
             params = {}
 
-        try:
-            access_token = self.get_access_token()
-            params["access_token"] = access_token
-        except Exception,e:
-            raise e
-
         result = self.__rest_client.post(uri, data, params)
         return result
-    
+
     def put(self, uri, data, params=None):
         """
         Generic resource put
@@ -311,29 +280,53 @@ class MP(object):
         if params is None:
             params = {}
 
-        try:
-            access_token = self.get_access_token()
-            params["access_token"] = access_token
-        except Exception,e:
-            raise e
-
         result = self.__rest_client.put(uri, data, params)
         return result
-    
+
+    def delete(self, uri, params=None):
+        """
+        Generic resource delete
+        @param uri
+        @return json
+
+        """
+        if params is None:
+            params = {}
+
+        result = self.__rest_client.delete(uri, params)
+        return result
+
     ##################################################################################
     class __RestClient(object):
-        __API_BASE_URL = "https://api.mercadolibre.com"
+        __API_BASE_URL = "https://api.mercadopago.com"
         MIME_JSON = "application/json"
         MIME_FORM = "application/x-www-form-urlencoded"
-
+        AUTH_HEADER = "Authorization"
         def __init__(self, outer):
             self.__outer = outer
             self.USER_AGENT = "MercadoPago Python SDK v"+self.__outer.version
+            self.PRODUCT_ID = "bc32bpftrpp001u8nhlg"
+            self.TRACKING_ID = "platform:"+str(sys.version_info.major)+"|"+platform.python_version()+",type:SDK"+self.__outer.version+",so;"
+
+        def get_mercadopago_transport_adapter(self):
+            """Creates and returns the transport adaptor for MP"""
+            return MPSSLAdapter()
+
+        def get_session(self):
+            """Creates and returns a ready-to-use requests.Session, with all the
+            customizations made to access MP
+            """
+            session = requests.Session()
+            session.mount(self.__API_BASE_URL,
+                          self.get_mercadopago_transport_adapter())
+            return session
+
+        def getAuthorizationHeader(self):
+            return 'Bearer ' + self.__outer.get_access_token()
 
         def get(self, uri, params=None):
-            s = requests.Session()
-            s.mount(self.__API_BASE_URL, MPSSLAdapter())
-            api_result = s.get(self.__API_BASE_URL+uri, params=params, headers={'User-Agent':self.USER_AGENT, 'Accept':self.MIME_JSON})
+            s = self.get_session()
+            api_result = s.get(self.__API_BASE_URL+uri, params=params, headers={'x-product-id': self.PRODUCT_ID, 'x-tracking-id': self.TRACKING_ID, 'User-Agent':self.USER_AGENT, 'Accept':self.MIME_JSON, self.AUTH_HEADER: self.getAuthorizationHeader()})
 
             response = {
                 "status": api_result.status_code,
@@ -346,9 +339,8 @@ class MP(object):
             if data is not None and content_type == self.MIME_JSON:
                 data = JSONEncoder().encode(data)
 
-            s = requests.Session()
-            s.mount(self.__API_BASE_URL, MPSSLAdapter())
-            api_result = s.post(self.__API_BASE_URL+uri, params=params, data=data, headers={'User-Agent':self.USER_AGENT, 'Content-type':content_type, 'Accept':self.MIME_JSON})
+            s = self.get_session()
+            api_result = s.post(self.__API_BASE_URL+uri, params=params, data=data, headers={'x-product-id': self.PRODUCT_ID, 'x-tracking-id': self.TRACKING_ID, 'User-Agent':self.USER_AGENT, 'Content-type':content_type, 'Accept':self.MIME_JSON, 'x-platform-id':self.__outer.platform_id, 'x-integrator-id':self.__outer.integrator_id, 'x-corporation-id':self.__outer.corporation_id, self.AUTH_HEADER: self.getAuthorizationHeader()})
 
             response = {
                 "status": api_result.status_code,
@@ -361,9 +353,19 @@ class MP(object):
             if data is not None and content_type == self.MIME_JSON:
                 data = JSONEncoder().encode(data)
 
-            s = requests.Session()
-            s.mount(self.__API_BASE_URL, MPSSLAdapter())
-            api_result = s.put(self.__API_BASE_URL+uri, params=params, data=data, headers={'User-Agent':self.USER_AGENT, 'Content-type':content_type, 'Accept':self.MIME_JSON})
+            s = self.get_session()
+            api_result = s.put(self.__API_BASE_URL+uri, params=params, data=data, headers={'x-product-id': self.PRODUCT_ID, 'x-tracking-id': self.TRACKING_ID, 'User-Agent':self.USER_AGENT, 'Content-type':content_type, 'Accept':self.MIME_JSON, self.AUTH_HEADER: self.getAuthorizationHeader()})
+
+            response = {
+                "status": api_result.status_code,
+                "response": api_result.json()
+            }
+
+            return response
+
+        def delete(self, uri, params=None):
+            s = self.get_session()
+            api_result = s.delete(self.__API_BASE_URL+uri, params=params, headers={'x-product-id': self.PRODUCT_ID, 'x-tracking-id': self.TRACKING_ID, 'User-Agent':self.USER_AGENT, 'Accept':self.MIME_JSON, self.AUTH_HEADER: self.getAuthorizationHeader()})
 
             response = {
                 "status": api_result.status_code,
